set more 1



*Parameters for model
local gamma=3
local minall=-0.5
local maxall=1.5


*Set periods
local samplestart=197301
local firstpred=199001
local lastpred=201412





*Load short interest data
import excel using Returns_short_interest_data.xlsx, firstrow clear sheet("Short interest")
qui destring _all, force replace
rename Date yyyymm
sort yy
save temp, replace




*Load other data and determine date for variable time (t=1,...,T)
import excel using Returns_short_interest_data.xlsx, firstrow clear sheet("GW variables")
destring _all, force replace
sort yy
merge m:m yy using temp
drop if _merge==2
drop _merge
sort yy



gen time=_n
tsset time
sum time if yy==`samplestart'
local tsamplestart=r(mean)
sum time if yy==`firstpred'
local tfirstpred=r(mean)
sum time if yy==`lastpred'
local tlastpred=r(mean)
gen month=mod(yy,100)



*Define variables

gen lnewsi=ln(EWSI)



gen ret=(Index+D12/12)/Index[_n-1]-1
replace ret=CRSP_SPvw if CRSP_SPvw!=.

gen exret=ret-Rfree[_n-1]
gen riskfree=Rfree[_n-1]

* Multi-period returns 

gen rf1=riskfree
gen rf3=(1+riskfree)*(1+riskfree[_n+1])*(1+riskfree[_n+2])-1
gen rf6= (1+riskfree)*(1+riskfree[_n+1])*(1+riskfree[_n+2])*(1+riskfree[_n+3])*(1+riskfree[_n+4])*(1+riskfree[_n+5])-1
gen rf12= (1+riskfree)*(1+riskfree[_n+1])*(1+riskfree[_n+2])*(1+riskfree[_n+3])*(1+riskfree[_n+4])*(1+riskfree[_n+5])*(1+riskfree[_n+6])*(1+riskfree[_n+7])*(1+riskfree[_n+8])*(1+riskfree[_n+9])*(1+riskfree[_n+10])*(1+riskfree[_n+11]) -1

gen r1=exret
gen r3=(1+ret)*(1+ret[_n+1])*(1+ret[_n+2])-(1+rf3)
gen r6=(1+ret)*(1+ret[_n+1])*(1+ret[_n+2])*(1+ret[_n+3])*(1+ret[_n+4])*(1+ret[_n+5]) - (1+rf6)
gen r12=(1+ret)*(1+ret[_n+1])*(1+ret[_n+2])*(1+ret[_n+3])*(1+ret[_n+4])*(1+ret[_n+5])*(1+ret[_n+6])*(1+ret[_n+7])*(1+ret[_n+8])*(1+ret[_n+9])*(1+ret[_n+10])*(1+ret[_n+11]) - (1+rf12)

gen svar1=svar
gen svar3=svar+svar[_n+1]+svar[_n+2]
gen svar6=svar3+svar3[_n+3]
gen svar12=svar6+svar6[_n+6]


foreach j in 1 3 6 12 {
	gen spred`j'=.	
	gen n`j'=.
}




*Produce out-of-sample forecasts

	


forvalues i=`tfirstpred'/`tlastpred'{
di `i'
	qui{
	capture drop zx*
	*detrend short index
	reg lnewsi time if yy>=`samplestart' & time<`i'
	predict zxsii, res
	
	
	foreach j in 1 3 6 12 {
		*with simple returns
		capture drop x
		reg r`j' l.zxsii if yy>`samplestart' & time<`i'+1-`j'
		predict x
		replace spred`j'=x if time==`i'
	}
	


	}
	
}

*Generate variance and means


foreach j in 1 3 6 12 {
	gen smodelvar`j'=.
	gen smodelmean`j'=.
}

forvalues i=`tsamplestart'/`tlastpred'{
		
		
		foreach j in 1 3 6 12 {
		
		
		sum r`j' if yy>=`samplestart' & time<`i'+1-`j'
		replace smodelmean`j'=r(mean) if time==`i'
				
		
		*- `j'+1 because this is how it's coded in Rapach Ringgenberg Zhou
		sum r`j' if time>=`i'-120 -`j'+1 & time<`i'+1-`j'
		replace smodelvar`j'=r(Var) if time==`i'
     		
		}	
		
		
}



*Performance analysis

matrix df = (1, -1, -`gamma'/2, `gamma'/2)
local z=1

foreach j in 1 3 6 12 {	
	foreach var of varlist spred`j' smodelmean`j'{
		gen wopt`var'= max(`minall',min(`maxall',`var'/`gamma'/smodelvar`j'))
		gen opt`var'= wopt`var'*r`j'+rf`j'
	}
	gen woptbuyhold`j'=1
	gen optbuyhold`j'=woptbuyhold`j'*r`j'+rf`j'

	
	
*Determine Certainty Equivalents and Sharpe ratios


local vars `" "optspred" "optsmodelmean" "optbuyhold" "'
 
foreach var of local vars{
	sum `var'`j' if yy>=`firstpred' & time<=`tlastpred'+1-`j' & int((month+`j'-1)/`j')==(month+`j'-1)/`j'
	local cee`var'`j'=r(mean)-0.5*`gamma'*r(Var)
	local m`var'`j'=r(Var)
	local n=r(N)
	
	gen ru`var'`j'=`var'`j'-0.5*`gamma'*w`var'`j'^2*svar`j'
	
	

	
	
	capture gen sharpe`var'=.
	gen e`var'`j'=`var'`j'-rf`j'
	sum e`var'`j' if yy>=`firstpred' & time<=`tlastpred'+1-`j' & int((month+`j'-1)/`j')==(month+`j'-1)/`j'
	local s1`var'=r(Var)
	replace sharpe`var'=r(mean)/r(sd)*(12/`j')^0.5 in `z'
	
	
	local z1=`z'+1
	srtest eoptspred`j' e`var'`j' if yy>=`firstpred' & time<=`tlastpred'+1-`j' & int((month+`j'-1)/`j')==(month+`j'-1)/`j'
	replace sharpe`var'=r(p) in `z1'
	
	
	cor optspred`j' `var'`j' if yy>=`firstpred' & yy<=`lastpred' & time<=`tlastpred'+1-`j' & int((month+`j'-1)/`j')==(month+`j'-1)/`j', cov
	local c=_result(4)
	
	capture gen du`var'=.
		
	replace du`var'=(`ceeoptspred`j''-`cee`var'`j'')*(1200/`j') in `z'
	
	matrix  omega = (`moptspred`j'',`c',0,0 \ `c',`m`var'`j'',0,0 \ 0,0,2*`moptspred`j''^2,2*`c'^2 \ 0,0,2*`c'^2,2*`m`var'`j''^2)
		
	matrix p=df*omega*df'	
	replace du`var'=2*normprob(-abs(du`var'[`z']/(1200/`j')/(p[1,1])^.5*`n'^0.5)) in `z1'
	
	
	
	local z2=`z'+2
	ttest ruoptspred`j'=ru`var'`j' if yy>=`firstpred' & yy<=`lastpred' & time<=`tlastpred'+1-`j' & int((month+`j'-1)/`j')==(month+`j'-1)/`j'
	replace du`var'=r(p) in `z2'
	
		
	
	
	
}


local z=`z'+5

}






gen what="Short Interest h=1" in 1
replace  what="Short Interest h=3" in 6
replace  what="Short Interest h=6" in 11
replace  what="Short Interest h=12" in 16

export excel what du* sharpe* using Paperresults.xlsx in 1/25, sheetmod sheet("Table2") cell(A1) first(var)

